﻿local AceOO = AceLibrary("AceOO-2.0")

local Borders = AceOO.Class()
local Quad = AceOO.Class()

function Quad.prototype:init(borders, xFlag, yFlag)
	Quad.super.prototype.init(self)

	self.xFlag = xFlag
	self.yFlag = yFlag

	local f = borders.frame
	local w = f:GetWidth()
	local h = f:GetHeight()

	local t = f:CreateTexture(nil, "BACKGROUND")
	t:SetWidth(w/2) 
	t:SetHeight(h/2)
	t:SetPoint("TOPLEFT", xFlag*w/2, -yFlag*h/2)

	self.borders = borders
	self.texture = t

	if xFlag == 1 then
		if yFlag == 1 then
			self.SetTexCoords = function (xc, xyc, yc) t:SetTexCoord(0,0, 0,yc, xc,0, xyc,xyc) end
		else
			self.SetTexCoords = function (xc, xyc, yc) t:SetTexCoord(0,yc, 0,0, xyc,xyc, xc,0) end
		end
	else
		if yFlag == 1 then
			self.SetTexCoords = function (xc, xyc, yc) t:SetTexCoord(xc,0, xyc,xyc, 0,0, 0,yc) end
		else
			self.SetTexCoords = function (xc, xyc, yc) t:SetTexCoord(xyc,xyc, xc,0, 0,yc, 0,0) end
		end
	end
end

function Quad.prototype:SetAmounts(x, xy, y)
	local xc = self.borders:GetCoordinate(x)
	local xyc = self.borders:GetCoordinate(xy)
	local yc = self.borders:GetCoordinate(y)

	self.SetTexCoords(xc, xyc, yc)
end

function Borders.prototype:init()
	Borders.super.prototype.init(self)

	local w = UIParent:GetWidth()
	local h = UIParent:GetHeight()

	local f = CreateFrame("Frame", nil, UIParent)
	f:SetFrameStrata("BACKGROUND")
	f:SetFrameLevel(1)
	f:SetPoint("TOPLEFT", -1, 1)
	f:SetPoint("BOTTOMRIGHT", 1, -1)
	f:Hide()
	self.frame = f

	self.topLeft = Quad(self, 0,0)
	self.topRight = Quad(self, 1,0)
	self.bottomLeft = Quad(self, 0,1)
	self.bottomRight = Quad(self, 1,1)
	self.quads = { self.topLeft, self.topRight, self.bottomLeft, self.bottomRight }

	self.emptyDistance = 0.5
end

function Borders.prototype:GetCoordinate(amount)
	-- 0: ed
	-- 1: 1
	local ed = self.emptyDistance
	local r = amount + ed*(1-amount)
	Bio_Tint:Debug("coord", amount..":"..r)
	return r
end

function Borders.prototype:SetBlendMode(mode)
	for _, quad in pairs(self.quads) do
		quad.texture:SetBlendMode(mode)
	end
end

function Borders.prototype:SetVertexColor(r,g,b)
	for _, quad in pairs(self.quads) do
		quad.texture:SetVertexColor(r,g,b)
	end
end

function Borders.prototype:SetTexture(texture, emptyDistance)
	self.emptyDistance = emptyDistance
	for _, quad in pairs(self.quads) do
		quad.texture:SetTexture(texture)
	end
end

function Borders.prototype:SetAmounts(b,br,r,tr,t,tl,l,bl)
	if l <= 0 and r <= 0 and b <= 0 and t <= 0 and br <= 0 and tr <= 0 and bl <= 0 and tl <= 0 then
		Bio_Tint:Debug("hiding border frame")
		self.frame:Hide()
	else
		self.topLeft:SetAmounts(l,tl,t)
		self.topRight:SetAmounts(r,tr,t)
		self.bottomLeft:SetAmounts(l,bl,b)
		self.bottomRight:SetAmounts(r,br,b)
		if not self.frame:IsShown() then
			Bio_Tint:Debug("showing border frame")
			self.frame:Show()
		end
	end
end

function Borders.prototype:SetAmount(a)
	self:SetAmounts(a,a,a,a,a,a,a,a)
end

Bio_Tint.Borders = Borders
